@@ -1,10 +1,8 @@
-use crate::future::Future;
-use crate::task::{Context, Poll};
-
-use std::io;
 use std::pin::Pin;
 
-use futures_io::AsyncWrite;
+use crate::future::Future;
+use crate::io::{self, Write};
+use crate::task::{Context, Poll};
 
 #[doc(hidden)]
 #[allow(missing_debug_implementations)]
@@ -12,7 +10,7 @@ pub struct FlushFuture<'a, T: Unpin + ?Sized> {
     pub(crate) writer: &'a mut T,
 }
 
-impl<T: AsyncWrite + Unpin + ?Sized> Future for FlushFuture<'_, T> {
+impl<T: Write + Unpin + ?Sized> Future for FlushFuture<'_, T> {
     type Output = io::Result<()>;
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
