@@ -178,7 +178,7 @@
 //! produce a stream. What gives?
 //!
 //! There's a trait in the standard library for converting something into an
-//! stream: [`IntoStream`]. This trait has one method, [`into_stream],
+//! stream: [`IntoStream`]. This trait has one method, [`into_stream`],
 //! which converts the thing implementing [`IntoStream`] into a stream.
 //!
 //! Unlike `std::iter::IntoIterator`, `IntoStream` does not have compiler
@@ -302,6 +302,7 @@
 
 pub use empty::{empty, Empty};
 pub use from_fn::{from_fn, FromFn};
+pub use from_iter::{from_iter, FromIter};
 pub use once::{once, Once};
 pub use repeat::{repeat, Repeat};
 pub use repeat_with::{repeat_with, RepeatWith};
@@ -313,6 +314,7 @@ pub(crate) mod stream;
 
 mod empty;
 mod from_fn;
+mod from_iter;
 mod once;
 mod repeat;
 mod repeat_with;
@@ -330,7 +332,7 @@ cfg_unstable! {
 
     pub use double_ended_stream::DoubleEndedStream;
     pub use exact_size_stream::ExactSizeStream;
-    pub use extend::Extend;
+    pub use extend::{extend, Extend};
     pub use from_stream::FromStream;
     pub use fused_stream::FusedStream;
     pub use interval::{interval, Interval};
