@@ -36,13 +36,11 @@ where
         let item = futures_core::ready!(Pin::new(&mut *self.stream).poll_next(cx));
 
         match item {
-            Some(v) => match (&mut self.p)(&v) {
-                true => Poll::Ready(Some(v)),
-                false => {
-                    cx.waker().wake_by_ref();
-                    Poll::Pending
-                }
-            },
+            Some(v) if (&mut self.p)(&v) => Poll::Ready(Some(v)),
+            Some(_) => {
+                cx.waker().wake_by_ref();
+                Poll::Pending
+            }
             None => Poll::Ready(None),
         }
     }
