@@ -1,21 +1,29 @@
 use std::marker::PhantomData;
 use std::pin::Pin;
 
+use pin_project_lite::pin_project;
+
 use crate::stream::Stream;
 use crate::task::{Context, Poll};
 
-/// A stream to skip elements of another stream based on a predicate.
-#[derive(Debug)]
-pub struct SkipWhile<S, P, T> {
-    stream: S,
-    predicate: Option<P>,
-    __t: PhantomData<T>,
+pin_project! {
+    /// A stream to skip elements of another stream based on a predicate.
+    ///
+    /// This `struct` is created by the [`skip_while`] method on [`Stream`]. See its
+    /// documentation for more.
+    ///
+    /// [`skip_while`]: trait.Stream.html#method.skip_while
+    /// [`Stream`]: trait.Stream.html
+    #[derive(Debug)]
+    pub struct SkipWhile<S, P, T> {
+        #[pin]
+        stream: S,
+        predicate: Option<P>,
+        __t: PhantomData<T>,
+    }
 }
 
 impl<S, P, T> SkipWhile<S, P, T> {
-    pin_utils::unsafe_pinned!(stream: S);
-    pin_utils::unsafe_unpinned!(predicate: Option<P>);
-
     pub(crate) fn new(stream: S, predicate: P) -> Self {
         SkipWhile {
             stream,
@@ -32,15 +40,16 @@ where
 {
     type Item = S::Item;
 
-    fn poll_next(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
+    fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
+        let mut this = self.project();
         loop {
-            let next = futures_core::ready!(self.as_mut().stream().poll_next(cx));
+            let next = futures_core::ready!(this.stream.as_mut().poll_next(cx));
 
             match next {
-                Some(v) => match self.as_mut().predicate() {
+                Some(v) => match this.predicate {
                     Some(p) => {
                         if !p(&v) {
-                            *self.as_mut().predicate() = None;
+                            *this.predicate = None;
                             return Poll::Ready(Some(v));
                         }
                     }
